@@ -2,7 +2,10 @@ package org.gcube.datacatalogue.grsf_manage_widget.client.view;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
+import java.util.Map.Entry;
 
 import org.gcube.datacatalogue.grsf_manage_widget.client.GRSFManageWidgetService;
 import org.gcube.datacatalogue.grsf_manage_widget.client.GRSFManageWidgetServiceAsync;
@@ -12,6 +15,7 @@ import org.gcube.datacatalogue.grsf_manage_widget.shared.ex.NoGRSFRecordExceptio
 
 import com.github.gwtbootstrap.client.ui.AlertBlock;
 import com.github.gwtbootstrap.client.ui.Button;
+import com.github.gwtbootstrap.client.ui.CheckBox;
 import com.github.gwtbootstrap.client.ui.ControlGroup;
 import com.github.gwtbootstrap.client.ui.Form;
 import com.github.gwtbootstrap.client.ui.Icon;
@@ -51,7 +55,22 @@ public class ManageProductWidget extends Composite{
 	AlertBlock infoBlock;
 
 	@UiField
-	TextArea nameTextArea;
+	TextArea shortNameTextBox;
+
+	@UiField
+	TextBox productGrsfType;
+	
+	@UiField
+	CheckBox traceabilityFlag;
+	
+	@UiField
+	TextArea GRSFNameTexBox;
+	
+	@UiField
+	TextArea semanticIdentifierTextBox;
+
+	@UiField
+	TextBox sources;
 
 	@UiField
 	TextBox currentStatus;
@@ -75,10 +94,13 @@ public class ManageProductWidget extends Composite{
 	ControlGroup listBoxStatusGroup;
 
 	@UiField
-	Form formUpdate;
+	ControlGroup annotationAreaGroup;
 
 	@UiField
-	TextBox productType;
+	ControlGroup productGrsfTypeGroup;
+
+	@UiField
+	Form formUpdate;
 
 	@UiField
 	Image loadingImage;
@@ -90,8 +112,8 @@ public class ManageProductWidget extends Composite{
 	protected static final String ERROR_ON_RETRIEVING_BEAN = "It seems there was a problem while contacting the service...";
 	protected static final String NO_GRSF_RECORD_BEAN = "This item is not a GRSF record, thus it cannot be managed";
 
-	protected static final String ERROR_ON_ROLE_CHECK = "Sorry but the service was not able to check if you have the rights to manage an item."
-			+ " You are suggested to contact the VRE Manager.";
+	protected static final String NO_ADMIN_ROLE = "Sorry but it seems you do not have the rights to manage items."
+			+ " You are suggested to contact the VRE Manager if something is wrong with this";
 	private ManageProductBean bean;
 
 	public ManageProductWidget(String productIdentifier) {
@@ -107,10 +129,11 @@ public class ManageProductWidget extends Composite{
 		// start loader service 
 		loadingImage.setUrl(LOADING_IMAGE_URL);
 		loadingImage.setVisible(true);
+		formUpdate.setVisible(false);
+		manageProductModal.show();
 
 		// async request to fetch the product
 		retrieveProductBean(productIdentifier);
-		manageProductModal.show();
 	}
 
 	private void retrieveProductBean(final String productIdentifier) {
@@ -122,9 +145,9 @@ public class ManageProductWidget extends Composite{
 			public void onSuccess(Boolean result) {
 
 				if(!result){
-					
-					showInfo(ERROR_ON_ROLE_CHECK, AlertType.ERROR);
-					
+
+					showInfo(NO_ADMIN_ROLE, AlertType.ERROR);
+
 					// hide the form and disable the send button
 					formUpdate.setVisible(false);
 					confirmButton.setEnabled(false);
@@ -140,11 +163,26 @@ public class ManageProductWidget extends Composite{
 								bean = result;
 								annotationArea.setText("");
 								infoBlock.setVisible(false);
-								nameTextArea.setText(bean.getProductName());
+								formUpdate.setVisible(true);
+								shortNameTextBox.setText(bean.getShortName());
 								currentStatus.setText(bean.getCurrentStatus().toString());
-								productType.setText(bean.getProductType());
+								productGrsfType.setText(bean.getGrsfType());
+								GRSFNameTexBox.setText(bean.getGrsfName());
+								semanticIdentifierTextBox.setText(bean.getSemanticId());
+								sources.setText(bean.getSources());
+								traceabilityFlag.setValue(bean.isTraceabilityFlag());
+
+								// check if we need to show more
+								if(bean.getExtrasIfAvailable() != null && !bean.getExtrasIfAvailable().isEmpty())
+									addExtrasAfter(bean, productGrsfTypeGroup);
+
 								List<GRSFStatus> statusToShow = new ArrayList<GRSFStatus>(STATUS);
 								statusToShow.remove(bean.getCurrentStatus());
+								
+								// if the record isn't approved, then remove also archived
+								if(!bean.getCurrentStatus().equals(GRSFStatus.Approved))
+									statusToShow.remove(GRSFStatus.Archived);
+								
 								listBoxStatus.addItem("Select the new status");
 								listBoxStatus.getElement().<SelectElement>cast().getOptions().getItem(0).setDisabled(true);
 								for (GRSFStatus availableStatus : statusToShow) {
@@ -181,9 +219,9 @@ public class ManageProductWidget extends Composite{
 
 			@Override
 			public void onFailure(Throwable caught) {
-				
-				showInfo(ERROR_ON_ROLE_CHECK, AlertType.ERROR);
-				
+
+				showInfo(NO_ADMIN_ROLE, AlertType.ERROR);
+
 				// hide the form and disable the send button
 				formUpdate.setVisible(false);
 				confirmButton.setEnabled(false);
@@ -194,6 +232,29 @@ public class ManageProductWidget extends Composite{
 
 	}
 
+	/**
+	 * Add extras if available after controlGroupBefore
+	 * @param bean
+	 * @param productTypeGroup
+	 */
+	private void addExtrasAfter(ManageProductBean bean,
+			ControlGroup controlGroupBefore) {
+
+		int index = formUpdate.getWidgetIndex(controlGroupBefore);
+
+		Map<String, String> extras = bean.getExtrasIfAvailable();
+		Iterator<Entry<String, String>> iterator = extras.entrySet().iterator();
+		while (iterator.hasNext()) {
+			Map.Entry<java.lang.String, java.lang.String> entry = (Map.Entry<java.lang.String, java.lang.String>) iterator
+					.next();
+
+			formUpdate.insert(new FormEntryModel(entry.getKey(), entry.getValue()), index);
+			index++;
+
+		}
+
+	}
+
 	@UiHandler("cancelButton")
 	void onCancelButton(ClickEvent ce){
 		manageProductModal.hide();
@@ -212,6 +273,14 @@ public class ManageProductWidget extends Composite{
 			return;
 		}
 
+		annotationAreaGroup.setType(ControlGroupType.NONE);
+
+		if(annotationArea.getText() == null || annotationArea.getText().isEmpty()){
+			annotationArea.setPlaceholder("An annotation message to send along the update (mandatory)");
+			annotationAreaGroup.setType(ControlGroupType.ERROR);
+			return;
+		}
+
 		manageProductModal.setCloseVisible(false);
 		cancelButton.setEnabled(false);
 		confirmButton.setEnabled(false);
